Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Configuration
Imports System.Net.NetworkInformation



Namespace Samples.Services.FinanceInfo
	Public Class FinanceInfoService
		Implements IFinanceInfoService

		''' <summary>
		''' No special initialization is required here
		''' </summary>
		Public Sub New()
		End Sub


		#Region "IFinanceInfoService (as data)"

		''' <summary>
		''' Get quotes for the specified list of symbols 
		''' </summary>
		''' <param name="symbols">Comma-separated list of stock symbols to retrieve</param>
		''' <param name="isOffline">Indicates whether offline or live data should be retrieved</param>
		''' <returns>Array of StockInfo</returns>
		Public Function GetQuotes(ByVal symbols As String, ByVal isOffline As Boolean) As StockInfo() Implements IFinanceInfoService.GetQuotes
			Dim provider As IFinanceInfoProvider = ResolveProvider(isOffline)
			If provider Is Nothing Then
				Throw New NullReferenceException("Invalid Provider specified.")
			End If

			Return provider.GetQuotes(symbols)
		End Function

		''' <summary>
		''' Get quotes for the specified list of symbols 
		''' </summary>
		''' <param name="symbols">Comma-separated list of stock symbols to retrieve</param>
		''' <returns>Array of StockInfo</returns>
		Public Function GetQuotes(ByVal symbols As String) As StockInfo() Implements IFinanceInfoService.GetQuotes
			Return GetQuotes(symbols, True)
		End Function

		''' <summary>
		''' Get quotes for the list of symbols stored in the web.config file of the host 
		''' </summary>
		''' <param name="isOffline">Indicates whether offline or live data should be retrieved</param>
		''' <returns>Array of StockInfo</returns>
		Public Function GetQuotes(ByVal isOffline As Boolean) As StockInfo() Implements IFinanceInfoService.GetQuotes
			Dim entry As String = (If(isOffline, FinanceInfoUtils.CONFIG_OFFLINESTOCKSYMBOLS, FinanceInfoUtils.CONFIG_ONLINESTOCKSYMBOLS))
			Dim symbols As String = ConfigurationManager.AppSettings(entry)
			Return GetQuotes(symbols, isOffline)
		End Function

		''' <summary>
		''' Get quotes for the list of symbols stored in the web.config file of the host 
		''' </summary>
		''' <returns>Array of StockInfo</returns>
		Public Function GetQuotes() As StockInfo() Implements IFinanceInfoService.GetQuotes
			Dim symbols As String = ConfigurationManager.AppSettings(FinanceInfoUtils.CONFIG_OFFLINESTOCKSYMBOLS)
			Return GetQuotes(symbols)
		End Function
		#End Region


		#Region "IFinanceInfoService (as HTML)"

		''' <summary>
		''' Get quotes for the specified list of symbols and returns HTML
		''' </summary>
		''' <param name="symbols">Comma-separated list of stock symbols to retrieve</param>
		''' <param name="isOffline">Indicates whether offline or live data should be retrieved</param>
		''' <returns>HTML message with current quotes</returns>
		Public Function GetQuotesAsHtml(ByVal symbols As String, ByVal isOffline As Boolean) As String Implements IFinanceInfoService.GetQuotesAsHtml
			Dim provider As IFinanceInfoProvider = ResolveProvider(isOffline)
			If provider Is Nothing Then
				Throw New NullReferenceException("Invalid Provider specified.")
			End If

			Return provider.GetQuotesAsHtml(symbols)
		End Function

		''' <summary>
		''' Get quotes for the list of symbols stored in the web.config file of the host and returns HTML
		''' </summary>
		''' <param name="isOffline">Indicates whether offline or live data should be retrieved</param>
		''' <returns>HTML message with current quotes</returns>
		Public Function GetQuotesAsHtml(ByVal isOffline As Boolean) As String Implements IFinanceInfoService.GetQuotesAsHtml
			Dim entry As String = (If(isOffline, FinanceInfoUtils.CONFIG_OFFLINESTOCKSYMBOLS, FinanceInfoUtils.CONFIG_ONLINESTOCKSYMBOLS))
			Dim symbols As String = ConfigurationManager.AppSettings(entry)
			Return GetQuotesAsHtml(symbols, isOffline)
		End Function

		''' <summary>
		''' Accepts a boolean parameter expressed as a string. Designed for being used with DynamicPopulate
		''' </summary>
		''' <param name="contextKey">The string FALSE if you want live data; everything else for offline data</param>
		''' <returns>HTML message with current quotes</returns>
		Public Function GetQuotesAsHtml(ByVal contextKey As String) As String Implements IFinanceInfoService.GetQuotesAsHtml
			' Parameter is transformed into a Boolean (if possible).
			' The string indicates the value for "isOffline" but has to default to TRUE.
			' If parsing fails, instead, the value is FALSE.
			Dim isOffline As Boolean = True
			If String.Equals(contextKey, "false", StringComparison.OrdinalIgnoreCase) Then
				isOffline = False
			End If

			Dim entry As String = (If(isOffline, FinanceInfoUtils.CONFIG_OFFLINESTOCKSYMBOLS, FinanceInfoUtils.CONFIG_ONLINESTOCKSYMBOLS))
			Dim symbols As String = ConfigurationManager.AppSettings(entry)
			Return GetQuotesAsHtml(symbols, isOffline)
		End Function

		#End Region


		#Region "Helpers:: Resolvers"

		''' <summary>
		''' Returns an instance of the class that provides data to the service
		''' </summary>
		''' <param name="isOffline">Indicates whether offline or live data should be retrieved</param>
		''' <returns>Instance of a class that implements IFinanceInfoProvider</returns>
		Protected Overridable Function ResolveProvider(ByVal isOffline As Boolean) As IFinanceInfoProvider
			Dim provider As IFinanceInfoProvider = Nothing

			If (Not isOffline) AndAlso Me.IsConnectedToInternet() Then
				provider = New OnlineServiceProvider()
			Else
				provider = New OfflineServiceProvider()
			End If

			Return provider
		End Function


		''' <summary>
		''' Ping a host and determine whether or not there is Internet connectivity
		''' </summary>
		''' <returns>Boolean</returns>
		Private Function IsConnectedToInternet() As Boolean
			Dim result As Boolean = False
			Dim p As New Ping()
			Dim reply As PingReply = p.Send("www.google.com", 3000)
			If reply.Status = IPStatus.Success Then
				Return True
			End If

			Return result
		End Function

		#End Region
	End Class
End Namespace


